/**
 * @typedef {import('ipfs-core-types').IPFS} IPFS
 * @typedef {import('multiformats/cid').CID} CID
 * @typedef {import('ipfs-message-port-protocol/error').EncodedError} EncodedError
 * @typedef {import('ipfs-message-port-protocol/cid').EncodedCID} EncodedCID
 * @typedef {import('ipfs-message-port-protocol/block').EncodedRmResult} EncodedRmResult
 * @typedef {import('ipfs-core-types/src/block').PutOptions} PutOptions
 */
export class BlockService {
    /**
     * @param {IPFS} ipfs
     */
    constructor(ipfs: IPFS);
    ipfs: import("ipfs-core-types").IPFS<any>;
    /**
     * @typedef {object} GetResult
     * @property {Uint8Array} block
     * @property {Set<Transferable>} transfer
     *
     * @typedef {object} GetQuery
     * @property {EncodedCID} cid
     * @property {number} [timeout]
     * @property {AbortSignal} [query.signal]
     *
     * @param {GetQuery} query
     * @returns {Promise<GetResult>}
     */
    get(query: {
        cid: EncodedCID;
        timeout?: number | undefined;
        signal?: AbortSignal | undefined;
    }): Promise<{
        block: Uint8Array;
        transfer: Set<Transferable>;
    }>;
    /**
     * @typedef {object} PutResult
     * @property {EncodedCID} cid
     * @property {Set<Transferable>} transfer
     *
     * @typedef {object} PutQuery
     * @property {Uint8Array} block
     * @property {EncodedCID|undefined} [cid]
     *
     * Stores input as an IPFS block.
     *
     * @param {PutOptions & PutQuery} query
     * @returns {Promise<PutResult>}
     */
    put(query: import("ipfs-core-types/src/block").PutOptions & {
        block: Uint8Array;
        /**
         * Stores input as an IPFS block.
         */
        cid?: EncodedCID | undefined;
    }): Promise<{
        cid: EncodedCID;
        transfer: Set<Transferable>;
    }>;
    /**
     * @typedef {object} RmQuery
     * @property {EncodedCID[]} cids
     * @property {boolean} [force]
     * @property {boolean} [quiet]
     * @property {number} [timeout]
     * @property {AbortSignal} [signal]
     *
     * Remove one or more IPFS block(s).
     * @param {RmQuery} query
     * @returns {Promise<EncodedRmResult[]>}
     */
    rm(query: {
        cids: EncodedCID[];
        force?: boolean | undefined;
        quiet?: boolean | undefined;
        timeout?: number | undefined;
        /**
         * Remove one or more IPFS block(s).
         */
        signal?: AbortSignal | undefined;
    }): Promise<EncodedRmResult[]>;
    /**
     * @typedef {object} StatQuery
     * @property {EncodedCID} cid
     * @property {number} [timeout]
     * @property {AbortSignal} [signal]
     *
     * @typedef {object} EncodedStatResult
     * @property {EncodedCID} cid
     * @property {number} size
     *
     * Gets information of a raw IPFS block.
     *
     * @param {StatQuery} query
     * @returns {Promise<EncodedStatResult>}
     */
    stat(query: {
        cid: EncodedCID;
        timeout?: number | undefined;
        signal?: AbortSignal | undefined;
    }): Promise<{
        cid: EncodedCID;
        /**
         * Gets information of a raw IPFS block.
         */
        size: number;
    }>;
}
export type IPFS = import('ipfs-core-types').IPFS;
export type CID = import('multiformats/cid').CID;
export type EncodedError = import('ipfs-message-port-protocol/error').EncodedError;
export type EncodedCID = import('ipfs-message-port-protocol/cid').EncodedCID;
export type EncodedRmResult = import('ipfs-message-port-protocol/block').EncodedRmResult;
export type PutOptions = import('ipfs-core-types/src/block').PutOptions;
//# sourceMappingURL=block.d.ts.map